import { defineStore } from 'pinia'
import { steeringAuthInit } from '@/stores/auth'

import {
  APICreate,
  APIListPaginated,
  APIRemove,
  APIRetrieve,
  APIUpdate,
  createExtendableAPI,
  createSteeringURL,
} from '@/api'

export type Note = {
  id: number
  title: string
  slug: string
  summary: string
  content: string
  imageId: number | null
  cbaId: number | null
  contributorIds: number[]
  links: { type: string; url: string }[]
  ownerId: number
  playlist: string
  tags: string
  timeslotId: number
  createdAt: string
  createdBy: string
  updatedAt: string
  updatedBy: string
}
type ReadonlyAttrs = 'id' | 'createdAt' | 'createdBy' | 'updatedAt' | 'updatedBy' | 'ownerId'
export type NewNote = Omit<Note, ReadonlyAttrs>
type NoteCreateData = Omit<Note, ReadonlyAttrs>
type NoteUpdateData = Partial<Omit<Note, ReadonlyAttrs>>

export function newNote(timeslotId: number): NewNote {
  return {
    title: '',
    slug: '',
    summary: '',
    content: '',
    imageId: null,
    cbaId: null,
    contributorIds: [],
    links: [],
    timeslotId,
    tags: '',
    playlist: '',
  }
}

export const useNoteStore = defineStore('notes', () => {
  const endpoint = createSteeringURL.prefix('notes')
  const { base, ...api } = createExtendableAPI<Note>(endpoint, steeringAuthInit)
  return {
    ...base,
    ...APIListPaginated(api),
    ...APIRetrieve(api),
    ...APICreate<Note, NoteCreateData>(api),
    ...APIUpdate<Note, NoteUpdateData>(api),
    ...APIRemove(api),
  }
})
